@@ -605,13 +605,13 @@ class H5DataIO(DataIO):
                        int with the registered filter ID, e.g. 307
         :type filter: String, int
         :param allow_plugin_filters: bool indicating whether the given filter can be dynamically loaded
-        :return: bool indicating wether the given filter is available
+        :return: bool indicating whether the given filter is available
         """
         if filter is not None:
             if filter in h5py_filters.encode:
                 return True
             elif allow_plugin_filters is True:
-                if type(filter) == int:
+                if isinstance(filter, int):
                     if h5py_filters.h5z.filter_avail(filter):
                         filter_info = h5py_filters.h5z.get_filter_info(filter)
                         if filter_info == (h5py_filters.h5z.FILTER_CONFIG_DECODE_ENABLED +
