@@ -33,13 +33,13 @@ class TestLabelledDict(TestCase):
     def test_set_key_attr(self):
         """Test that the key attribute cannot be set after initialization."""
         ld = LabelledDict(label='all_objects')
-        with self.assertRaisesWith(AttributeError, "can't set attribute"):
+        with self.assertRaises(AttributeError):
             ld.key_attr = 'another_name'
 
     def test_getitem_unknown_val(self):
         """Test that dict[unknown_key] where the key unknown_key is not in the dict raises an error."""
         ld = LabelledDict(label='all_objects', key_attr='prop1')
-        with self.assertRaisesWith(KeyError, "'unknown_key'"):
+        with self.assertRaises(KeyError):
             ld['unknown_key']
 
     def test_getitem_eqeq_unknown_val(self):
@@ -161,7 +161,7 @@ class TestLabelledDict(TestCase):
         ld = LabelledDict(label='all_objects', key_attr='prop1')
         obj1 = MyTestClass('a', 'b')
         ld.add(obj1)
-        with self.assertRaisesWith(KeyError, "'unknown_val'"):
+        with self.assertRaises(KeyError):
             ld['prop1 == unknown_val']  # same as ld['unknown_val']
 
     def test_getitem_eqeq_nonempty_unknown_attr(self):
